# -*- mode: python; coding: utf-8 -*-
from __future__ import print_function


__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import csv
import logging
import os.path

class PhotoDB:
    def __init__(self, config):

        self._db_file = config.db_file()
        self._output_dir = config.output_dir()
        self._file_mode = config.output_chmod()
        self._hashes = {}
        self.load()

    def load(self, merge=False, filename=None):
        """
        loads an existing DB

        If 'merge' is True, the values of previously loaded DBs (if any)
        are kept.
        If the path of a file is passed in 'filename', then
        """

        if filename is None:
            filename = self._db_file

        if not merge:
            self._hashes = {}
        try:
            logging.info("----------")
            logging.info("DB Loading %s" % filename)
            with open(filename, 'r', encoding='utf-8') as f_in:
                dbreader = csv.reader(f_in, delimiter=',')
                try:
                    names = next(dbreader)
                except StopIteration:
                    logging.info("DB was empty")
                    return

                for file_dir, file_name, file_type, hash in dbreader:
                    self._hashes[hash] = {'dir': file_dir,
                                              'name': file_name,
                                              'type': file_type}
            logging.info("DB Load finished, %d entries" % len(self._hashes))
        except IOError as e:
            if e.errno==2:
                logging.debug("DB file %s doesn't exist " % self._db_file + \
                    "yet, it will get created")
            else:
                logging.error("Error opening DB file %s" % self._db_file)
                raise

    def write(self):

        try:
            os.remove(self._db_file+".bak")
        except:
            pass

        try:
            os.rename(self._db_file, self._db_file+".bak",self._file_mode)
        except:
            pass

        with open(self._db_file, 'w', encoding='utf-8') as f_out:

            dbwriter = csv.writer(f_out, delimiter=',')
            dbwriter.writerow(['directory', 'filename', 'type', 'md5'])

            for hash in self._hashes.keys():

                file_dir, file_name, file_type = (
                    self._hashes[hash]['dir'],
                    self._hashes[hash]['name'],
                    self._hashes[hash]['type'])

                dbwriter.writerow([file_dir, file_name, file_type, hash])

    def add_to_db(self, file_dir, file_name, media_file):
        try:
            hash = media_file.hash()
        except IOError as e:
            logging.error("IOError %s trying to hash %s" %
                          (e,media_file.get_path()))
            return False

        file_type = media_file.type()

        # remove output dir path + '/'
        file_dir = file_dir[len(self._output_dir) + 1:]
        self._hashes[hash] = {'dir': file_dir,
                              'name': file_name,
                              'type': file_type}

        logging.info("indexed %s/%s %s %s" % (file_dir,
                                              file_name,
                                              file_type,
                                              hash))
        return True

    def is_duplicate(self, media_file):
        """
        checks if the given file has been already sorted
        returns True if so, False if not
        """
        hash = media_file.hash()

        if hash in self._hashes:

            filename_data = self._hashes[hash]
            filename2 = self._output_dir + "/" + filename_data['dir']+'/'+filename_data['name']

            if not media_file.is_equal_to(filename2):
                logging.critical("MD5 hash collision for two different files,"
                                 "handled as dupe: %s %s", media_file.get_path(), filename2)

            logging.info("%s was detected as duplicate with %s" % (media_file.get_path(), filename2) )

            return True
        return False



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import photosort.test
from photosort import media
import shutil
import tempfile
import os
import stat

class TestPhotoMedia(photosort.test.TestCase):

    def setUp(self):
        self.img1 = self.get_data_path('media1/img1.jpg')
        self.img1dup = self.get_data_path('media1/img1_dup.jpg')
        self.photo = media.MediaFile.build_for(self.img1)
        
    def test_hash_creation(self):
        expected_hash = "a35de42abad366d0f6232a4abd0404c8 - 2013-08-24 13:05:52"
        self.assertEqual(self.photo.hash(),expected_hash)

        # check for hasher non being re-started
        same_photo = media.MediaFile.build_for(self.img1)
        self.assertEqual(same_photo.hash(),expected_hash)

    def test_datetime(self):
        expected_datetime = "2013-08-24 13:05:52" # it must come from exif data

        self.assertEqual(str(self.photo.datetime()),expected_datetime)

    def test_equal_checking(self):
        self.assertTrue(self.photo.is_equal_to(self.img1dup))

    def test_datatime_dir(self):
        dir_fmt = '%(year)d/%(year)04d_%(month)02d_%(day)02d'
        dir_str = self.photo.calculate_datetime(dir_fmt)
        self.assertEqual(dir_str,"2013/2013_08_24")

    def test_get_filename(self):
        self.assertEqual(self.photo.get_filename(),'img1.jpg')

    def test_rename(self):
        tmpdir = tempfile.gettempdir()
        tmpfile = tmpdir + '/' + self.photo.get_filename()
        tmpfile_renamed = tmpdir + '/R' + self.photo.get_filename()
        tmpfile_mode = 0o666

        shutil.copy(self.photo.get_path(), tmpfile)

        photo_t = media.MediaFile.build_for(tmpfile)

        photo_t.rename_as(tmpfile_renamed,tmpfile_mode)
        self.assertTrue(self.photo.is_equal_to(tmpfile_renamed))

        file_mode = os.stat(tmpfile_renamed)[stat.ST_MODE] & 0o777
        self.assertEqual(file_mode,tmpfile_mode)

    def test_move_to_directory(self):
        tmpdir = tempfile.gettempdir()
        tmpfile = tmpdir + '/' + self.photo.get_filename()
        dir_fmt = '%(year)d/%(year)04d_%(month)02d_%(day)02d'
        file_fmt = '%(year)04d%(month)02d%(day)02d%(hour)02d%(minute)02d%(second)02d_'

        shutil.copy(self.photo.get_path(), tmpfile)
        photo_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(tmpdir+'/2013',ignore_errors=True)
        photo_t.move_to_directory_with_date(tmpdir,dir_fmt)

        self.assertTrue(self.photo.is_equal_to(tmpdir+'/2013/2013_08_24/img1.jpg'))

        shutil.copy(self.photo.get_path(), tmpfile)
        photo_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(tmpdir+'/2013',ignore_errors=True)
        photo_t.move_to_directory_with_date(tmpdir,dir_fmt, file_fmt)

        self.assertTrue(self.photo.is_equal_to(tmpdir+'/2013/2013_08_24/20130824130552_img1.jpg'))

if __name__ == '__main__':
    unittest.main()
   



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import photosort.test

class TestQAWorks(photosort.test.TestCase):

    def setUp(self):
        self.dummy = None

    def test_assert_true( self ):
        self.assertTrue( True )

    def test_assert_equal( self ):
        self.assertEqual(2, 1+1)


if __name__ == '__main__':
    unittest.main()
   


# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import unittest
import os.path
import platform

if platform.python_version() < '2.7':
    unittest = __import__('unittest2')
else:
    import unittest

class TestCase(unittest.TestCase):
    def get_data_path(self,file_path):
        return os.path.join(os.path.dirname(__file__),'data',file_path)



# -*- mode: python; coding: utf-8 -*-
from __future__ import print_function

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import datetime
import os
import logging
import fcntl
import time

import media


class WalkForMedia:
    """
        A simple class to walk for JPEGs over a root dir
    """
    def __init__(self, rootdir, ignores=[], extensions=[]):
        self._rootdir = rootdir
        self._ignores = ignores
        self._extensions = extensions
        self._fs_time_skew = self._fs_timeskew_to(rootdir)

    def _fs_timeskew_to(self,rootdir):
        """
        discover the remote filesystem time skew with local datetime
        this could be handled by ntp syncing all nodes, but we
        can't have a guarantee on this
        """
        f_name = os.path.join(rootdir,".timesync")
        with open(f_name,'w') as f:
            f.write("touch!")

        ct1 = os.path.getmtime(f_name)
        ct2 = os.path.getctime(f_name)

        os.remove(f_name) # cleanup the file

        # it can differ from windows to UN*X
        ct =max(ct1,ct2)

        now = time.mktime(time.gmtime())

        return ct-now # remote-local

    def _modification_lapse(self,filename):
        """
        return the lapse from last file modification (in seconds)
        """
        ct1 = os.path.getmtime(filename)
        ct2 = os.path.getctime(filename)

        # it can differ from windows to UN*X
        ct = max(ct1,ct2)

        now = time.mktime(time.gmtime())

        return now-ct + self._fs_time_skew

    def _file_is_growing(self,filename):
        size1 = os.path.getsize(filename)
        time.sleep(2)
        size2 = os.path.getsize(filename)

        if size2>size1:
            logging.debug("%s size1<2 = %d<%d",filename,size1,size2)

        return size2>size1

    def _file_is_empty(self,filename):
        return os.path.getsize(filename) == 0

    def _file_is_locked(self,filename):
        try:
            with open(filename,'r') as file:
                fcntl.flock(file.fileno(),fcntl.LOCK_EX)
                fcntl.flock(file.fileno(),fcntl.LOCK_UN)
        except IOError as e:
                logging.debug("%s seems to be locked or gone" % filename)
                return True
        return False # we were able to lock/unlock, so nobody must be writing

    def _file_is_ready(self,filename):
        # we have a bunch of extra checks to avoid files
        # that are yet incomplete from being moved around

        if self._file_is_locked(filename):
            logging.debug("file %s not ready because it's locked"
                          % filename )
            return False

        # skip files that were modified in the last 30 seconds

        mod_lapse = self._modification_lapse(filename)

        if mod_lapse<30:
            logging.debug("file %s not ready because modification "
                          "lapse is %d, it's probably copying yet"
                          % (filename,mod_lapse) )
            return False

        # skip growing or empty files

        # skip this check, as it's a major slowdown, and lapse seems to work
        #if self._file_is_growing(filename):
        #     logging.debug("file %s not ready because it's growing"
        #                  % filename )
        #     return False

        if self._file_is_empty(filename):
            logging.debug("file %s not ready because it's empty"
                          % filename )
            return False


        return True

    def find_media(self):

        if not os.path.isdir(self._rootdir):
            logging.info(self._rootdir +
                         " does not exists or it's not mounted, "
                         "cannot find media")
            return

        if os.path.split(self._rootdir)[-1].startswith('.'):
            logging.info(self._rootdir +
                         " is a hidden directory => ignoring")
            return

        if self._rootdir in self._ignores:
            logging.info(self._rootdir +
                         " in the list to be ignored => ignoring")
            return

        for root, subFolders, files in os.walk(self._rootdir):

            subFolders[:] = [sf for sf in subFolders
                    if not sf.startswith('.') 
                    and not sf in self._ignores]

            for file in files:

                file_path = os.path.join(root, file)

                # skip mac osx AppleDouble files (it puts a ._ in front of the
                # name) to keep extra information

                if file.startswith('.'):
                    continue

                media_file = media.MediaFile.build_for(file_path)
                file_type = media_file.type()

                if file_type != 'unknown':
                    if self._file_is_ready(file_path):
                       yield [root, file]



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import unittest
import platform
import os.path

if platform.python_version() < '2.7':
    unittest = __import__('unittest2')
else:
    import unittest


def main():
    tests_dir = os.path.join(os.path.dirname(__file__),'testcases')
    testsuite = unittest.TestLoader().discover(tests_dir,pattern="*.py")
    unittest.TextTestRunner(verbosity=100).run(testsuite)

if __name__ == '__main__':
    main()




# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import photosort.test
from photosort import walk

class TestWalkForMedia(photosort.test.TestCase):

    def setUp(self):
        self.media1 = self.get_data_path('media1')
        
    def test_directory_inspection(self):
        walker = walk.WalkForMedia(self.media1)
        files = [file for root, file in walker.find_media()]
        self.assertTrue('img1.jpg' in files)

    def test_ignores(self):
        pass

if __name__ == '__main__':
    unittest.main()
   


# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import photosort.test
from photosort import media
import shutil
import tempfile
import os
import stat
import time

class TestMovieMedia(photosort.test.TestCase):

    def setUp(self):
        self.mov1 = self.get_data_path('media2/mov1.mp4')
        self.mov1dup = self.get_data_path('media2/mov1_dup.mp4')
        self.mov1_mtime = os.path.getmtime(self.mov1)
        self.movie = media.MediaFile.build_for(self.mov1)

    def test_hash_creation(self):
        expected_hash = "d41d8cd98f00b204e9800998ecf8427e"
        self.assertEqual(self.movie.hash(),expected_hash)

        # check for hasher non being re-started
        same_movie = media.MediaFile.build_for(self.mov1)
        self.assertEqual(same_movie.hash(),expected_hash)

    def test_datetime(self):
        mtime = time.localtime(self.mov1_mtime)
        expected_datetime = time.strftime("%Y-%m-%d %H:%M:%S", mtime)# it must come from exif data
        decimals = "%.6g" % (round(self.mov1_mtime % 1, 6))
        try:
            ms = decimals.split('.')[1]
            expected_datetime += '.' + ms
        except IndexError:
            pass
        self.assertEqual(str(self.movie.datetime()),expected_datetime)

    def test_equal_checking(self):
        self.assertTrue(self.movie.is_equal_to(self.mov1dup))

    def test_datatime_dir(self):
        mtime = time.localtime(self.mov1_mtime)
        year = "%d" % mtime[0]
        month = "%02d" % mtime[1]
        day = "%02d" % mtime[2]

        dir_fmt = '%(year)d/%(year)04d_%(month)02d_%(day)02d'
        dir_str = self.movie.calculate_datetime(dir_fmt)
        expected_dir = os.path.join(year, year+'_'+month+'_'+day)
        self.assertEqual(dir_str,expected_dir)

    def test_get_filename(self):
        self.assertEqual(self.movie.get_filename(),'mov1.mp4')

    def test_rename(self):

        tmpdir = tempfile.gettempdir()
        tmpfile = tmpdir + '/' + self.movie.get_filename()
        tmpfile_renamed = tmpdir + '/R' + self.movie.get_filename()
        tmpfile_mode = 0o666

        shutil.copy(self.movie.get_path(), tmpfile)

        movie_t = media.MediaFile.build_for(tmpfile)

        movie_t.rename_as(tmpfile_renamed,tmpfile_mode)
        self.assertTrue(self.movie.is_equal_to(tmpfile_renamed))

        file_mode = os.stat(tmpfile_renamed)[stat.ST_MODE] & 0o777
        self.assertEqual(file_mode,tmpfile_mode)

    def test_move_to_directory(self):
        tmpdir = tempfile.gettempdir()
        tmpfile = tmpdir + '/' + self.movie.get_filename()
        dir_fmt = '%(year)d/%(year)04d_%(month)02d_%(day)02d'
        file_fmt = '%(year)04d%(month)02d%(day)02d%(hour)02d%(minute)02d%(second)02d_'

        shutil.copy(self.movie.get_path(), tmpfile)
        mov_mtime = os.path.getmtime(tmpfile)
        mtime = time.localtime(mov_mtime)
        year = "%d" % mtime[0]
        month = "%02d" % mtime[1]
        day = "%02d" % mtime[2]
        hour = "%02d" % mtime[3]
        minute = "%02d" % mtime[4]
        second = "%02d" % mtime[5]
        movie_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(tmpdir+'/'+year,ignore_errors=True)
        movie_t.move_to_directory_with_date(tmpdir,dir_fmt)

        expected_filename = os.path.join(tmpdir, year, year+'_'+month+'_'+day, 'mov1.mp4')
        self.assertTrue(self.movie.is_equal_to(expected_filename))

        shutil.copy(self.movie.get_path(), tmpfile)
        mov_mtime = os.path.getmtime(tmpfile)
        mtime = time.localtime(mov_mtime)
        year = "%d" % mtime[0]
        month = "%02d" % mtime[1]
        day = "%02d" % mtime[2]
        hour = "%02d" % mtime[3]
        minute = "%02d" % mtime[4]
        second = "%02d" % mtime[5]
        movie_t = media.MediaFile.build_for(tmpfile)

        shutil.rmtree(tmpdir+'/'+year,ignore_errors=True)
        movie_t.move_to_directory_with_date(tmpdir,dir_fmt,file_fmt)

        expected_filename = os.path.join(tmpdir, year, year+'_'+month+'_'+day, year+month+day+hour+minute+second+'_mov1.mp4')
        self.assertTrue(self.movie.is_equal_to(expected_filename))

if __name__ == '__main__':
    unittest.main()
   



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"


class Error(Exception):
    """Base class for exceptions in this module."""
    pass


class DirError(Error):
    """Exception raised for errors with directories.

    Attributes:
        expr -- input expression in which the error occurred
        msg  -- explanation of the error
    """

    def __init__(self, expr, msg):
        self.expr = expr
        self.msg = msg



# -*- mode: python; coding: utf-8 -*-


__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import argparse
import logging
import traceback

import sys
import time
import photodb
import config
import walk
import os
import media

class PhotoSort:

    def __init__(self, config_filename, log_level):

        self._config = config.Config(config_filename)
        self._setup_logging(log_level)
        self._photodb = photodb.PhotoDB(self._config)
        self._duplicates_dir = self._config.duplicates_dir()
        self._dir_pattern = self._config.dir_pattern()
        self._file_prefix = self._config.file_prefix()
        self._inputs = (self._config.sources()[source]['dir']
                        for source in self._config.sources().keys())
        self._file_mode = self._config.output_chmod()

    def _setup_logging(self, log_level):
        if self._config.log_file():
            logging.basicConfig(filename=self._config.log_file(),
                                level=log_level)
        else:
            logging.basicConfig(stream=sys.stderr,
                                level=log_level)

    def _sync_source(self,src_dir):
        try:
            walker = walk.WalkForMedia(src_dir)
        except IOError:
            logging.error("Unable to walk dir %s", src_dir)
            return

        for file_dir,file_name in walker.find_media():
            file_path = os.path.join(file_dir,file_name)
            media_file = media.MediaFile.build_for(file_path)
            if self._photodb.is_duplicate(media_file):
                file = media_file.get_filename()
                duplicates_path = os.path.join(self._duplicates_dir,file)

                logging.info("moving to duplicates: %s" %
                     duplicates_path)

                media_file.rename_as(duplicates_path,self._file_mode)
            else:
                if media_file.move_to_directory_with_date(self._photodb._output_dir,
                                                     self._dir_pattern,
                                                     self._file_prefix,
                                                     self._file_mode):
                    self._photodb.add_to_db(media_file.get_directory(), media_file.get_filename(), media_file)
        self._photodb.write()

    def rebuild_db(self):
        """
        registers in the DB the media files already existing in the
        target directory to be able to detect duplicates and avoid
        overwritting
        """
        walker = walk.WalkForMedia(self._config.output_dir(), ignores=self._inputs)
        for file_dir,file_name in walker.find_media():
            try:
                file_path = os.path.join(file_dir, file_name)
                media_file = media.MediaFile.build_for(file_path)
                self._photodb.add_to_db(file_dir, file_name, media_file)
            except:
                logging.critical("Unexpected error: %s" % (sys.exc_info()[0]))
        self._photodb.write()

    def sync(self):
        """
        ensures that the media files of the input directories are sorted
        """
        for source,value in self._config.sources().items():
            self._sync_source(value['dir'])

    def monitor(self):
        """
        regularly (10s at the time of this writting)
        ensures that the media files of the input directories are sorted
        """
        while True:
            self.sync()
            time.sleep(10)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('op', action="store",
                        choices=['sync', 'rebuilddb', 'monitor'],
                        help="Operation")

    group = parser.add_argument_group('Common parameters')
    group.add_argument('--config', action="store",
                       help="Customized configuration file",
                       default='/etc/photosort.yml')
    group.add_argument('--debug',
                       action="store_true",
                       help="Enable debugging")
    ns = parser.parse_args()

    log_level = logging.INFO
    if ns.debug:
        log_level = logging.DEBUG
    photo_sort = PhotoSort(ns.config, log_level)

    try:
        if ns.op == "sync":
            photo_sort.sync()

        elif ns.op == "rebuilddb":
            photo_sort.rebuild_db()

        elif ns.op == "monitor":
            photo_sort.monitor()
        else:
            print("Unknown operation: %s" % ns.op)
    except:
        logging.critical("Unexpected error: %s" % (sys.exc_info()[0]))
        logging.critical(traceback.format_exc())


if __name__ == "__main__":
        main()



# -*- mode: python; coding: utf-8 -*-
from __future__ import print_function

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

from PIL import Image
from PIL.ExifTags import TAGS
import datetime
import sys
import time
import logging

import media

class Photo(media.MediaFile):

    def __init__(self, filename):
        media.MediaFile.__init__(self, filename)
        self.__exif_data = None

    def _exif_data(self):
        """Returns a dictionary from the exif data of an
         PIL Image item. """
        self.__exif_data = {}
        try:
            image = Image.open(self._filename)
        except IOError as e:
            if str(e).startswith("cannot identify image file"):
                return {}
            else:
                raise

        try:
            info = image._getexif()
        except (AttributeError, IndexError):
            return {}


        if info:
            for tag, value in info.items():
                decoded = TAGS.get(tag, tag)
                self.__exif_data[decoded] = value

        return self.__exif_data

    def _exif_datetime(self):
        exif_datetime_str = ""

        exif_data = self._exif_data()
        for exif_tag in ['DateTimeOriginal', 'Image DateTime', 'DateTimeDigitized']:
            try:
                exif_datetime_str = exif_data[exif_tag]
            except KeyError:
                logging.debug("EXIF tag not available: " + exif_tag)
                continue
            except IOError as e:
                if str(e) == "not enough data":
                    return None
                if str(e) == "cannot identify image file":
                    return None
                else:
                    raise
            except ValueError:
                return None  # time data '0000:00:00 00:00:00'
            # only reached if the datetime information properly obtained
            logging.debug("photo date and time obtained from: " + exif_tag)
            break

        if exif_datetime_str == '0000:00:00 00:00:00':
            return None

        if exif_datetime_str:
            try:
                return datetime.datetime.strptime(str(exif_datetime_str),
                                              '%Y:%m:%d %H:%M:%S')
            except UnicodeEncodeError as e:
                if str(e).startswith("'ascii' codec can't encode character"):
                    return None
                else:
                    raise
        else:
            return None

    def datetime(self):
        dt = self._exif_datetime()
        logging.debug("date and time: " + str(dt))
        if dt is None:
            dt = media.MediaFile.datetime(self)

        return dt

    def hash(self):
        """
        Builds an hexadecimal hash for a picture, extended with the
        EXIF date as a string, to prevent as much as possible from md5 collisions
        """
        if self._hash is not None:
            return self._hash

        media_hash = media.MediaFile.hash(self)
        exif_datetime = self._exif_datetime()

        if exif_datetime is not None:
            media_hash += " - " + str(exif_datetime)
        self._hash = media_hash
        return media_hash


if __name__ == "__main__":
    photo = Photo(sys.argv[1])
    print(photo)



# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"

import filecmp
import hashlib
import logging
import os.path
import os
import stat
import sys
import datetime
import shutil


class MediaFile:

    def __init__(self, filename):
        self._filename = filename
        self._file_type = MediaFile.guess_file_type(filename)
        self._hash = None

    @staticmethod
    def guess_file_type(filename):

        extension = filename.lower().split('.')[-1]
        if extension in ('jpeg', 'jpg', 'cr2', 'raw', 'png', 'arw', 'thm', 'orf'):
            return 'photo'
        if extension in ('mpeg', 'mpg', 'mov', 'mp4', 'avi'):
            return 'movie'
        return 'unknown'

    @staticmethod
    def build_for(filename):

        file_type = MediaFile.guess_file_type(filename)
        if file_type is 'photo':
            import photo    # delayed import to avoid circular dependencies
            return photo.Photo(filename)
        else:
            return MediaFile(filename)

    def get_filename(self):
        return os.path.basename(self._filename)

    def get_directory(self):
        return os.path.dirname(self._filename)

    def get_path(self):
        return self._filename

    def hash(self, hasher=None, blocksize=65536):
        if self._hash is not None:
            return self._hash

        if hasher is None:
            hasher = hashlib.md5()

        with open(self._filename, 'rb') as afile:
            buf = afile.read(blocksize)
            while len(buf) > 0:
                hasher.update(buf)
                buf = afile.read(blocksize)

            self._hash = hasher.hexdigest()
            return self._hash

    def datetime(self):

        ct1 = os.path.getmtime(self._filename)
        ct2 = os.path.getctime(self._filename)

        ct = min(ct1,ct2) # it can differ from windows to UN*X

        return datetime.datetime.fromtimestamp(ct)

    def __str__(self):

        s = "[%s file hash=%s date=%s]" % (self._file_type, self.hash(), self.datetime())
        return s

    def is_equal_to(self,filename):

        try:
            result = filecmp.cmp(self._filename, filename, shallow=True)
            return result
        except OSError as e:

            logging.info("Comparing to %s, file didn't exist anymore, erased or moved?" % filename )
            return False

    def type(self):
        return self._file_type

    def makedirs_f(self, path, mode):
        paths = os.path.split(path)

        total_path = ''
        for directory in paths:
            total_path = os.path.join(total_path,directory)
            if os.path.isdir(total_path):
                continue
            else:
                os.mkdir(total_path,mode | stat.S_IXUSR)


    def rename_as(self,new_filename,file_mode = 0o774):

        try:
            self.makedirs_f(os.path.dirname(new_filename),file_mode)
        except:
            logging.error("Unable to move: %s" % new_filename)
            return False

        try:
            result = shutil.move(self._filename, new_filename)
            os.chmod(new_filename,file_mode)
        except OSError as e:
            logging.error("Unable to move: %s" % e)
            return False

        except IOError as e:
            logging.error("Unable to move: %s" % e)
            return False

        except shutil.Error as e:
            logging.error("Unable to move: %s" % e)
            return False

        except:
            raise

        return True

    def calculate_datetime(self,format):
        dt = self.datetime()
        data = {'year': dt.year, 'month': dt.month, 'day': dt.day,
                'hour': dt.hour, 'minute': dt.minute, 'second': dt.second }

        return format % data

    def move_to_directory_with_date(self,directory,dir_format,file_format='',file_mode=0o774):

        out_dir = directory + "/" + self.calculate_datetime(dir_format)

        try:
            os.mkdir(out_dir)
            os.chmod(out_dir,file_mode | stat.S_IXUSR)
        except OSError as e:
            pass # it already exists

        if file_format:
            file_prefix = self.calculate_datetime(file_format) + self.get_filename()
        else:
            file_prefix = self.get_filename()
        new_filename = out_dir + "/" + file_prefix
        logging.info("moving %s to %s" % (self._filename, new_filename))

        if self.rename_as(new_filename, file_mode):
            self._filename = new_filename
            return True
        else:
            return False

if __name__ == "__main__":
    file = MediaFile.build_for(sys.argv[1])
    print(file)




# -*- mode: python; coding: utf-8 -*-

__author__ = "Miguel Angel Ajo Pelayo"
__email__ = "miguelangel@ajo.es"
__copyright__ = "Copyright (C) 2013 Miguel Angel Ajo Pelayo"
__license__ = "GPLv3"


import yaml


class Config:
    """
    YAML configuration reading class
    """
    def __init__(self, filename='/etc/photosort.yml'):
        with open(filename, 'r') as f_in:
            self._data = yaml.safe_load(f_in)

    def output_dir(self):
        return self._data['output']['dir']

    def _relative_or_absolute_to_output(self, filename):
        """
            If filename starts with a slash it does return
            the filename, otherwise it will add the output dir
            in front
        """
        if filename.startswith('/'):
            return filename
        else:
            return self.output_dir()+"/"+filename

    def sources(self):
        return self._data['sources']

    def log_file(self):
        if 'log_file' in self._data['output']:
            return self._relative_or_absolute_to_output(
                self._data['output']['log_file'])
        else:
            return None

    def db_file(self):
        return self._relative_or_absolute_to_output(self._data['output']['db_file'])

    def duplicates_dir(self):
        return self._relative_or_absolute_to_output(
            self._data['output']['duplicates_dir'])

    def dir_pattern(self):
        try:
            return self._data['output']['dir_pattern']
        except KeyError as exc:
            if str(exc) == "'dir_pattern'":
                return self._data['output']['pattern']
            else:
                raise

    def file_prefix(self):
        try:
            return self._data['output']['file_prefix']
        except KeyError as exc:
            if str(exc) == "'file_prefix'":
                return ""
            else:
                raise

    def output_chmod(self):
        return int(self._data['output']['chmod'],8) # octal conversion




#!/usr/bin/env python3

import setuptools

setuptools.setup(
        name = 'photosort',
        version = '2018.1',
        description = 'Picture inbox simplified',
        author = 'Miguel Angel Ajo Pelayo',
        author_email = 'miguelangel@ajo.es',
        url = 'https://github.com/mangelajo/photosort',
        packages = setuptools.find_packages(exclude = ['ez_setup']),
        include_package_data = True,
        zip_safe = False,
        entry_points = {
            'console_scripts': [
                'photosort = photosort.photosort:main'
            ]},
        install_requires = ['pyaml', 'Pillow'],
        data_files = [('etc', ['etc/photosort.yml'])],
        test_suite = 'photosort.test.testcases'
        )



